from setuptools import setup, find_packages


try:
    with open("README.md", "r", encoding="utf-8") as fh:
        long_description = fh.read()
except FileNotFoundError:
    print("README.md file not found.")
    long_description = ""
except Exception as e:
    print("An error occurred while reading README.md:", str(e))
    long_description = ""
setup(
    name="sirifi",
    version="0.1.5.2",
    author='Sagar Narwade, Rudra Desai',
    description="SiriFi: Smart Insights & Research for Investments in Financial Instruments",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/CodeEagle22/SiriFi",  # Change or remove if not applicable
    packages=find_packages(),
    install_requires=[
        "aiohappyeyeballs==2.6.1",
        "aiohttp==3.12.13",
        "aiosignal==1.4.0",
        "alpha_vantage==3.0.0",
        "anyio==4.9.0",
        "attrs==25.3.0",
        "backports.tarfile==1.2.0",
        "beautifulsoup4==4.13.4",
        "binance==0.3.55",
        "build==1.3.0",
        "certifi==2025.6.15",
        "cffi==1.17.1",
        "charset-normalizer==3.4.2",
        "contourpy==1.3.2",
        "cryptography==45.0.5",
        "curl_cffi==0.11.4",
        "cycler==0.12.1",
        "dateparser==1.2.2",
        "docutils==0.22",
        "dotenv==0.9.9",
        "fastjsonschema==2.21.2",
        "fonttools==4.58.5",
        "frozendict==2.4.6",
        "frozenlist==1.7.0",
        "greenlet==3.2.4",
        "h11==0.16.0",
        "httpcore==1.0.9",
        "httpx==0.28.1",
        "id==1.5.0",
        "jsonschema==4.25.1",
        "jsonschema-specifications==2025.9.1",
        "jaraco.classes==3.4.0",
        "jaraco.context==6.0.1",
        "jaraco.functools==4.2.1",
        "jedi==0.18.2",
        "markdown-it-py==3.0.0",
        "matplotlib==3.10.3",
        "mdurl==0.1.2",
        "more-itertools==10.7.0",
        "multidict==6.6.3",
        "multitasking==0.0.11",
        "narwhals==2.5.0",
        "nbformat==5.10.4",
        "newsapi==0.1.1",
        "newsapi-python==0.2.7",
        "nh3==0.3.0",
        "numpy==1.26.4",
        "openpyxl==3.1.5",
        "pandas==2.3.0",
        "patsy==1.0.1",
        "peewee==3.18.1",
        "pickleshare==0.7.5",
        "pillow==11.3.0",
        "playwright==1.54.0",
        "plotly==6.3.0",
        "praw==7.8.1",
        "prawcore==2.4.0",
        "propcache==0.3.2",
        "protobuf==6.31.1",
        "pycparser==2.22",
        "pycryptodome==3.23.0",
        "pyee==13.0.0",
        "Pygments==2.16.1",
        "pyparsing==3.2.3",
        "pyproject_hooks==1.2.0",
        "PySocks==1.7.1",
        "python-binance==1.0.29",
        "python-dotenv==1.1.1",
        "python-telegram-bot==22.3",
        "pytz==2025.2",
        "readme_renderer==44.0",
        "referencing==0.36.2",
        "regex==2024.11.6",
        "requests==2.32.4",
        "requests-futures==1.0.2",
        "requests-toolbelt==1.0.0",
        "rfc3986==2.0.0",
        "rich==14.1.0",
        "rpds-py==0.27.1",
        "schedule==1.2.2",
        "scipy==1.10.1",
        "seaborn==0.13.2",
        "selenium==4.34.0",
        "six==1.16.0",
        "sniffio==1.3.1",
        "sortedcontainers==2.4.0",
        "soupsieve==2.7",
        "statsmodels==0.13.5",
        "stocktrends==0.1.5",
        "ta==0.11.0",
        "tqdm==4.67.1",
        "trio==0.30.0",
        "trio-websocket==0.12.2",
        "twine==6.1.0",
        "typing_extensions==4.14.0",
        "tzdata==2025.2",
        "tzlocal==5.3.1",
        "update-checker==0.18.0",
        "urllib3==2.4.0",
        "vaderSentiment==3.3.2",
        "websocket-client==1.8.0",
        "websockets==15.0.1",
        "wsproto==1.2.0",
        "yahooquery==2.4.1",
        "yarl==1.20.1",
        "yfinance==0.2.64"
        
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
    python_requires='>=3.7',
    include_package_data=True,
)
